# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/models.autoencoders.ipynb.

# %% auto 0
__all__ = ['AutoEncoder', 'AutoEncoderPL']

# %% ../../nbs/models.autoencoders.ipynb 4
import torch.nn.functional as F
import torch.nn as nn
import torch
from ..modules import Encoder, Decoder
from pytorch_lightning import LightningModule
from ..image.datasets import MNISTDataset
from torch.utils.data import DataLoader

# %% ../../nbs/models.autoencoders.ipynb 5
class AutoEncoder(nn.Module):
    def __init__(self,
        encoder:Encoder, # Encoder layer
        decoder:Decoder # Decoder layer
        ):
        super().__init__()
        self.encoder = encoder
        self.decoder = decoder
    
    def forward(self,
        x:torch.Tensor # Tensor B x L
        )->torch.Tensor:
        z = self.encoder(x)
        x_hat = self.decoder(z)
        return x_hat

# %% ../../nbs/models.autoencoders.ipynb 8
class AutoEncoderPL(LightningModule):
    def __init__(self, autoencoder:AutoEncoder):
        super().__init__()
        # self.save_hyperparameters()
        self.save_hyperparameters(ignore=['autoencoder'])
        self.autoencoder = autoencoder
        self.metric = torch.nn.MSELoss()

    def forward(self, x):
        return self.autoencoder(x)
    
    def predict_step(self, batch, batch_idx):
        x, y = batch
        x = x.view(x.size(0), -1)
        with torch.no_grad():
            return self.autoencoder(x)

    def _shared_eval(self, batch, batch_idx, prefix, prog_bar=True, on_step=False, on_epoch=True, sync_dist=True):
        x, _ = batch
        x = x.view(x.size(0), -1) # flatten B x C x H x W to B x L (grey pic)
        x_hat = self.autoencoder(x)
        loss = self.metric(x_hat, x)
        self.log(f"{prefix}/loss", loss, on_step=on_step, on_epoch=on_epoch, sync_dist=sync_dist)
        return loss

    def training_step(self, batch, batch_idx):
        return self._shared_eval(batch, batch_idx, "train")
    
    def test_step(self, batch, batch_idx):
        self._shared_eval(batch, batch_idx, "test")
    
    def validation_step(self, batch, batch_idx):
        return self._shared_eval(batch, batch_idx, "val")

    def configure_optimizers(self):
        optimizer = torch.optim.Adam(self.parameters(), lr=1e-3)
        return optimizer

